"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DPI = exports.PageOrientation = exports.Size = exports.Unit = exports.Format = void 0;
const jspdf_1 = require("jspdf");
const file_saver_1 = require("file-saver");
const mapbox_gl_1 = require("mapbox-gl");
require("js-loading-overlay");
const fabric_1 = require("fabric");
exports.Format = {
    JPEG: 'jpg',
    PNG: 'png',
    PDF: 'pdf',
    SVG: 'svg',
};
exports.Unit = {
    in: 'in',
    mm: 'mm',
};
exports.Size = {
    LETTER: [279, 216],
    A2: [594, 420],
    A3: [420, 297],
    A4: [297, 210],
    A5: [210, 148],
    A6: [148, 105],
    B2: [707, 500],
    B3: [500, 353],
    B4: [353, 250],
    B5: [250, 176],
    B6: [176, 125],
};
exports.PageOrientation = {
    Landscape: 'landscape',
    Portrait: 'portrait',
};
exports.DPI = {
    72: 72,
    96: 96,
    200: 200,
    300: 300,
    400: 400,
};
class MapGenerator {
    constructor(map, size = exports.Size.A4, dpi = 300, format = exports.Format.PNG.toString(), unit = exports.Unit.mm, accesstoken) {
        this.map = map;
        this.width = size[0];
        this.height = size[1];
        this.dpi = dpi;
        this.format = format;
        this.unit = unit;
        this.accesstoken = accesstoken;
    }
    generate() {
        const this_ = this;
        JsLoadingOverlay.show({
            overlayBackgroundColor: '#5D5959',
            overlayOpacity: '0.6',
            spinnerIcon: 'ball-spin',
            spinnerColor: '#2400FD',
            spinnerSize: '2x',
            overlayIDName: 'overlay',
            spinnerIDName: 'spinner',
            offsetX: 0,
            offsetY: 0,
            containerID: null,
            lockScroll: false,
            overlayZIndex: 9998,
            spinnerZIndex: 9999,
        });
        const actualPixelRatio = window.devicePixelRatio;
        Object.defineProperty(window, 'devicePixelRatio', {
            get() { return this_.dpi / 96; },
        });
        const hidden = document.createElement('div');
        hidden.className = 'hidden-map';
        document.body.appendChild(hidden);
        const container = document.createElement('div');
        container.style.width = this.toPixels(this.width);
        container.style.height = this.toPixels(this.height);
        hidden.appendChild(container);
        const style = this.map.getStyle();
        if (style && style.sources) {
            const sources = style.sources;
            Object.keys(sources).forEach((name) => {
                const src = sources[name];
                Object.keys(src).forEach((key) => {
                    if (!src[key])
                        delete src[key];
                });
            });
        }
        const renderMap = new mapbox_gl_1.Map({
            accessToken: this.accesstoken || mapbox_gl_1.accessToken,
            container,
            style,
            center: this.map.getCenter(),
            zoom: this.map.getZoom(),
            bearing: this.map.getBearing(),
            pitch: this.map.getPitch(),
            interactive: false,
            preserveDrawingBuffer: true,
            fadeDuration: 0,
            attributionControl: false,
            transformRequest: this.map._requestManager._transformRequestFn,
        });
        const images = (this.map.style.imageManager || {}).images || [];
        Object.keys(images).forEach((key) => {
            renderMap.addImage(key, images[key].data);
        });
        renderMap.once('idle', () => {
            var _a;
            const canvas = renderMap.getCanvas();
            const fileName = `map.${this_.format}`;
            switch (this_.format) {
                case exports.Format.PNG:
                    this_.toPNG(canvas, fileName);
                    break;
                case exports.Format.JPEG:
                    this_.toJPEG(canvas, fileName);
                    break;
                case exports.Format.PDF:
                    this_.toPDF(renderMap, fileName);
                    break;
                case exports.Format.SVG:
                    this_.toSVG(canvas, fileName);
                    break;
                default:
                    console.error(`Invalid file format: ${this_.format}`);
                    break;
            }
            renderMap.remove();
            (_a = hidden.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(hidden);
            Object.defineProperty(window, 'devicePixelRatio', {
                get() { return actualPixelRatio; },
            });
            JsLoadingOverlay.hide();
        });
    }
    toPNG(canvas, fileName) {
        canvas.toBlob((blob) => {
            (0, file_saver_1.saveAs)(blob, fileName);
        });
    }
    toJPEG(canvas, fileName) {
        const uri = canvas.toDataURL('image/jpeg', 0.85);
        const a = document.createElement('a');
        a.href = uri;
        a.download = fileName;
        a.click();
        a.remove();
    }
    toPDF(map, fileName) {
        const canvas = map.getCanvas();
        const pdf = new jspdf_1.jsPDF({
            orientation: this.width > this.height ? 'l' : 'p',
            unit: this.unit,
            compress: true,
        });
        pdf.addImage(canvas.toDataURL('image/png'), 'png', 0, 0, this.width, this.height, undefined, 'FAST');
        const { lng, lat } = map.getCenter();
        pdf.setProperties({
            title: map.getStyle().name,
            subject: `center: [${lng}, ${lat}], zoom: ${map.getZoom()}`,
            creator: 'Mapbox GL Export Plugin',
            author: '(c)Mapbox, (c)OpenStreetMap',
        });
        pdf.save(fileName);
    }
    toSVG(canvas, fileName) {
        const uri = canvas.toDataURL('image/png');
        fabric_1.fabric.Image.fromURL(uri, (image) => {
            const tmpCanvas = new fabric_1.fabric.Canvas('canvas');
            const pxWidth = Number(this.toPixels(this.width, this.dpi).replace('px', ''));
            const pxHeight = Number(this.toPixels(this.height, this.dpi).replace('px', ''));
            image.scaleToWidth(pxWidth);
            image.scaleToHeight(pxHeight);
            tmpCanvas.add(image);
            const svg = tmpCanvas.toSVG({
                x: 0,
                y: 0,
                width: pxWidth,
                height: pxHeight,
                viewBox: {
                    x: 0,
                    y: 0,
                    width: pxWidth,
                    height: pxHeight,
                },
            });
            const a = document.createElement('a');
            a.href = `data:application/xml,${encodeURIComponent(svg)}`;
            a.download = fileName;
            a.click();
            a.remove();
        });
    }
    toPixels(length, conversionFactor = 96) {
        if (this.unit === exports.Unit.mm) {
            conversionFactor /= 25.4;
        }
        return `${conversionFactor * length}px`;
    }
}
exports.default = MapGenerator;
//# sourceMappingURL=map-generator.js.map